from lib2to3.pgen2 import driver
import time
import urllib.request
import os
import requests
import io

from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import NoSuchElementException       
from PIL import Image

import webdriver_management as wm

class GoogleImageScraper():
    def __init__(self, webdriver_path, image_path, search_key= 'Institut Teknologi Bandung', num_images= 1, headless= False, min_resolution=(0,0), max_resolution= (1920, 1080) ):
        # parameter types
        image_path = os.path.join(image_path, search_key)

        if (type(num_images)!= int):
            print('[ERROR] Number of images must be integer value.')
            return
        if not os.path.exists(image_path):
            print('[INFO] Image Folder not found!. Creating a new folder!')
            os.makedirs(image_path)

        # Check updated chromedriver

        while True:
            try:
                # visit www.google.com
                options = Options()
                if headless:
                    options.add_argument('--headless')
                driver = webdriver.Chrome(webdriver_path, chrome_options=options )
                driver.set_window_size(1920, 1800)
                driver.get('https://www.google.com')
                break

            except:
                # chromedriver not available or outdated
                try:
                    driver
                except NameError:
                    is_patched = wm.download_lastest_chromedriver()
                else:
                    is_patched = wm.download_lastest_chromedriver(driver.capabilities['version'])
                if (not is_patched):
                    exit('[ERROR] Please update the chromedriver.exe in the webdriver folder according to your chrome version:https://chromedriver.chromium.org/downloads')
        self.driver = driver
        self.search_key = search_key
        self.num_images = num_images
        self.webdriver_path = image_path
        self.url = 'https://www.google.com/search?q=%s&source=lnms&tbm=isch&sa=X&ved=2ahUKEwie44_AnqLpAhUhBWMBHUFGD90Q_AUoAXoECBUQAw&biw=1920&bih=947'%(search_key)
        self.headless = headless
        self.min_resolution = min_resolution
        self.max_resolution = max_resolution
    
    def find_image(self):

        print("[INFO] Scraping for image link... Please wait.")
        image_urls= []
        count= 0
        missed_count= 0
        self.driver.get(self.url)
        time.sleep(3)
        indx = 1
        while self.num_images > count:
            try:
                # find and click image
                img_url = self.driver.find_element_by_xpath('//*[@id="islrg"]/div[1]/div[%s]/a[1]/div[1]/img'%(str(indx)))
                img_url.click()
                missed_count = 0
            except Exception:
                if (missed_count>5):
                    print('[INFO] No more photos.')
                    break

                try:
                    #select image
                    time.sleep(1)
                    class_names= ['n3VNCb']
                    images = [self.driver.find_elements_by_class_name(class_name) for class_name in class_names if len(self.driver.find_elements_by_class_name(class_name)) != 0 ][0]
                    for image in images:
                        #only download images that starts with http
                        src_link = image.get_attribute("src")
                        if(("http" in  src_link) and (not "encrypted" in src_link)):
                            print("[INFO] %d. %s"%(count,src_link))
                            image_urls.append(src_link)
                            count +=1
                            break
                except Exception:
                    print("[INFO] Unable to get link")   
                    
                try:
                    #scroll page to load next image
                    if(count%3==0):
                        self.driver.execute_script("window.scrollTo(0, "+str(indx*60)+");")
                    element = self.driver.find_element_by_class_name("mye4qd")
                    element.click()
                    print("[INFO] Loading more photos")
                    time.sleep(3)
                except Exception:  
                    time.sleep(1)
                indx += 1

            
            self.driver.quit()
            print("[INFO] Google search ended")
            return image_urls

    def save_images(self,image_urls):
        #save images into file directory
        """
            This function takes in an array of image urls and save it into the prescribed image path/directory.
            Example:
                google_image_scraper = GoogleImageScraper("webdriver_path","image_path","search_key",number_of_photos)
                image_urls=["https://example_1.jpg","https://example_2.jpg"]
                google_image_scraper.save_images(image_urls)
                
        """
        print("[INFO] Saving Image... Please wait...")
        for indx,image_url in enumerate(image_urls):
            try:
                print("[INFO] Image url:%s"%(image_url))
                search_string = ''.join(e for e in self.search_key if e.isalnum())
                image = requests.get(image_url,timeout=5)
                if image.status_code == 200:
                    with Image.open(io.BytesIO(image.content)) as image_from_web:
                        try:
                            filename = "%s%s.%s"%(search_string,str(indx),image_from_web.format.lower())
                            image_path = os.path.join(self.image_path, filename)
                            print("[INFO] %d .Image saved at: %s"%(indx,image_path))
                            image_from_web.save(image_path)
                        except OSError:
                            rgb_im = image_from_web.convert('RGB')
                            rgb_im.save(image_path)
                        image_resolution = image_from_web.size
                        if image_resolution != None:
                            if image_resolution[0]<self.min_resolution[0] or image_resolution[1]<self.min_resolution[1] or image_resolution[0]>self.max_resolution[0] or image_resolution[1]>self.max_resolution[1]:
                                image_from_web.close()
                                #print("GoogleImageScraper Notification: %s did not meet resolution requirements."%(image_url))
                                os.remove(image_path)

                        image_from_web.close()
            except Exception as e:
                print("[ERROR] Failed to be downloaded",e)
                pass
        print("[INFO] Download Completed. Please note that some photos are not downloaded as it is not in the right format (e.g. jpg, jpeg, png)")